/**
 *
 *  Elevator.java
 *  
 *       @author	Russell Lowke
 *      @version	Last modified 10_15_03
 *
 *    CSCIE-160:	Assignment Two
 *
 *        Usage:	java Elevator
 *
 *
 *  Description:	Elevator simulation for a multi-story building.
 *
 *       Method:	Use boardPassenger(floorDest), to add each elevator passenger 
 *						with a floor destination of [floorDest], then use
 *						go() to start the elevator. go() will cause the elevator
 *    					to search for the nearest requested floor and drop off passengers for
 *   					that floor, then proceed to the next nearest requested destination until
 *						all passengers have been dropped off.
 *
 *        Notes:	Tab Width = 4
 *
 *					Variable prefix conventions:
 *						p 	stands for property of object, also known as a member variable or field
 *						m 	stands for "my" and means a local variable to a method
 *						Obj stands for an object from this package
 */


package cscie160.hw2;

public class Elevator {

	// Static vars
	public static final int CAPACITY   = 10;						// maximum number of people in evevator
	public static final int NUM_FLOORS = 7;							// # of floors in building
	
	
    // Private vars
	private int 	 pCurrentFloor;									// floor elevator is currently on
    private int 	 pTravelDirection;
    private int 	 pPassengers;									// # of pPassengers in elevator
    private int   [] pPassengerDest;								// # of pPassengers requesting each floor
    private int   [] pFloorRequests;								// destination requests for each floor
	private Floor [] objFloor;										// array of floor objects
	        
	public Elevator() {
		
		pCurrentFloor	   = 1;									
		pTravelDirection   = +1;									// 0 = stopped, 1 = up, -1 = down
		pPassengers		   = 0;									
		pPassengerDest	   = new   int[NUM_FLOORS + 1];				//   Note: pPassengerDest[0] unused			
		pFloorRequests	   = new   int[NUM_FLOORS + 1];				//   Note: pFloorRequests[0] unused
		objFloor		   = new Floor[NUM_FLOORS + 1];				//   Note: objFloor[0] unused
		
		// generate floors
		for (int i = 1; i <= NUM_FLOORS; i++ ) {
			objFloor[i] = new Floor(i);
		}
	}
	
   /**
	*
    *	Move elevator one floor in the pTravelDirection.
    *
    */
    public void move() {
        
        pCurrentFloor += pTravelDirection;
            
        // ensure floor is legal
        if (pCurrentFloor < 1) {
            pCurrentFloor    = 1;									// elevator at botton, so stop
            pTravelDirection = 0;
        }
        else if (pCurrentFloor > NUM_FLOORS) {
            pCurrentFloor    = NUM_FLOORS;							// elevator at top, so stop
            pTravelDirection = 0;
        }
		
		System.out.println("Elevator moves to floor " + pCurrentFloor);
    }
     
	 
   /**
    * 
    *	Board a passenger destined for a floor.
    * 
    *	@param	mDestinationFloor	The floor that passenger wants to go to.
    */
    public void boardPassenger(int mDestinationFloor) throws ElevatorFullException {
			
		if (pPassengers >= CAPACITY)
			throw new ElevatorFullException("Elevator is full");
			
		pPassengers++;												// add passenger to elevator
		pPassengerDest[mDestinationFloor]++;
		pFloorRequests[mDestinationFloor]++;
		
		System.out.println("Passenger boards elevator destined for floor " + mDestinationFloor);
    }
	
	
   /**
    * 
    *	Unload a passenger.
    *
    */
    public void unloadPassenger() {
        pPassengers--;
        pPassengerDest[pCurrentFloor]--;
		
		System.out.println("Passenger leaves elevator on floor " + pCurrentFloor);
    }
        
        
   /**
    * 
    *	Stop the elevator and unload pPassengers destined for this floor.
    *
    */
    public void stop() {
    
        pTravelDirection = 0;										// stop
        System.out.println("Elevator stops on floor " + (pCurrentFloor));
        
		objFloor[pCurrentFloor].unloadAndLoadPassengers(this);
        
		System.out.println(this);									// print new elevator state
    }
	
	
   /**
    * 
    *	Register a request for floor [mFloor].
    *
	*	@param		mFloor				Floor number being requested.
	*	@param		mCreateThatPerson	If true put a person on that floor waiting.
    */
	public void registerRequest(int mFloorNum, boolean mCreateThatPerson) {
	
		pFloorRequests[mFloorNum]++;								// set a request for floor mFloor
		
		if (mCreateThatPerson)										// if flag [mCreateThatPerson] true
			objFloor[mFloorNum].addPersonWaiting();					// create the person on the floor waiting
			
		System.out.println("Elevator requested for floor " + mFloorNum);
	}
	
	
   /**
    * 
    * 	Return the direction of closest destination floor,
    * 	with preference for moving down.
	*
	*	Note: the current floor is ignored, so if the elevator is full
	*		  and there are still people this floor waiting
	*		  a loop will not occur.
    *
	*		  If the elevator is full then floor requests should probably
	*		  be ignored and only floor destinations looked at. (for later)
	*
    *	@return		Direction of closest destination floor.
    */
    public int findDirection() {
        
        int mTargetDirection = 0;									// direction elevator should move
        int mModifier        = 1;									// mModifier to check for destination floor
        
		int mTestFloorUp;											// floor above being tested for
		int mTestFloorDwn;											// floor below being tested for
		
        while (mTargetDirection == 0 && mModifier < NUM_FLOORS) {	// loop until target found
            
            // floors to test 
            mTestFloorUp  = pCurrentFloor + mModifier;
            mTestFloorDwn = pCurrentFloor - mModifier;
            
            // keep tests within bounds
            if (mTestFloorUp > NUM_FLOORS)
                mTestFloorUp = NUM_FLOORS;
            if (mTestFloorDwn < 1)
                mTestFloorDwn = 1;
            
            // test for floor
            if (pFloorRequests[mTestFloorDwn] > 0)
                mTargetDirection = -1;								// going down
            else if (pFloorRequests[mTestFloorUp] > 0)
                mTargetDirection = +1;								// going up
            else
                mModifier++;										// check next
        }
        
        return mTargetDirection;
    }
 
 
   /**
    * 
    * 	Drive elevator to next destination.
    *
    */ 
    public void go() {
        
        pTravelDirection = findDirection();							// get new direction of travel
        
        while(pTravelDirection != 0) {
            move();													// move me
            if (pFloorRequests[pCurrentFloor] != 0) {				// check for floor requests on this floor
                stop();												// stop elevator and load/unload
                pTravelDirection = findDirection();					// get new direction of travel
            }    
        }
    }
    
        
   /**
    * 
    *	Override the toString() method of java.lang.Object for printing Elevator values.
	*
    *	@return		summary of pertinent values in elevartor.
    */
	public String toString() {
		String mReturn = "\n" + "Currently " + pPassengers + " passengers onboard \n" + 
                             "Current Floor: " + (pCurrentFloor) + "\n";
		return mReturn;
	}
        
     
	/**
    * 
    * 	Return true if elevator is at capacity.
	*	(no longer used, but a nice method to have around)
    *
    *	@return		true if elevator is at capacity.
    */
	public boolean isElevatorFull() {
	
		boolean mReturn = false;
		
		if (pPassengers == CAPACITY)
			mReturn = true;
			
		return mReturn;
	}
	
	
   /**
    * 
    * 	Return the # of passengers destined for floor [mFloor].
	*	Called by unloadAndLoadPassengers() in class Floor.
    *
	*	@param		mFloor	Floor number being checked for.
	*
    *	@return				# of passengers destined for floor [mFloor]
    */
	public int getNumPassengersDestForFloor(int mFloor) {
		return pPassengerDest[mFloor];
	}
	
	
   /**
    * 
    * 	Needed by loadPassengers() in class Floor to set the floor requests
	*	for floor [mFloor] = the number of passengers still on that floor
	*	waiting for the elevator [mNewValue] when elevator is full.
    *
	*	@param		mFloor		Floor number being checked for.
	*	@param		mNewValue	Floor number being checked for.
    */
	public void setFloorRequests(int mFloor, int mNewValue) {
		pFloorRequests[mFloor] = mNewValue;
	}
	
	      
   /**
    * 
    * 	Create an elevator and board two pPassengers into it.
    *
    */	
	static public void main(String mArgv[]) {

		
		Elevator mElevator = new Elevator();						// make an elevator
		System.out.println(mElevator);								// show starting state of elevator
		
		try {
			mElevator.boardPassenger(3);
			mElevator.boardPassenger(4);
			mElevator.boardPassenger(4);
			mElevator.boardPassenger(7);
			
			mElevator.registerRequest(3, true);
			mElevator.registerRequest(3, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
			mElevator.registerRequest(6, true);
		}
		catch(ElevatorFullException e) {
			System.out.println(e.getMessage());
		}
		
		mElevator.go();												// Do elevator thing.
		
    }
}

