 /**
 *
 *  Elevator.java
 *  
 *       @author	Russell Lowke
 *      @version	Last modified 11_03_03
 *
 *    CSCIE-160:	Assignment Three
 *
 *        Usage:	java Elevator
 *
 *
 *  Description:	Elevator simulation for a multi-story building.
 *
 *       Method:	Use newPerson(mOnFloor, mDestFloor), to add each a passenger 
 *						on floor [mOnFloor] with a floor destination of [mDestFloor], then use
 *						go() to start the elevator. go() will cause the elevator
 *    					to search for the nearest requested floor and drop off passengers for
 *   					that floor, then proceed to the next nearest requested destination until
 *						all passengers have been collected and dropped off.
 *
 *        Notes:	Tab Width = 4
 *
 *					Variable prefix conventions:
 *						p 	stands for property of object, also known as a member variable or field
 *						m 	stands for "my" and means a local variable to a method
 *						Obj stands for an object from this package
 */


package cscie160.hw3;

import java.util.*;

public class Elevator {

	// Static vars
	public  static final int CAPACITY   = 10;						// maximum number of people in evevator
	public  static final int NUM_FLOORS = 7;						// # of floors in building
	private static Floor  [] objFloor;								// array of floor objects
		
    // Private vars
	private ArrayList pPassengerList;								// list of Passenger objects in this Elevator
    private int    [] pFloorRequests;								// requests for each floor
	private int 	  pCurrentFloor;								// floor elevator is currently on
    private int 	  pTravelDirection;								// 0 = stopped, 1 = up, -1 = down
	
	public Elevator() {
		
		objFloor		   = new Floor[NUM_FLOORS + 1];				//   Note: objFloor[0] unused
		
		// generate floors
		for (int i = 1; i <= NUM_FLOORS; i++ ) {
			objFloor[i] = new Floor(i);
		}
		
		pCurrentFloor	   = 1;										// floor elevator is now on				
		pTravelDirection   = 0;										// 0 = stopped, 1 = up, -1 = down	
		pFloorRequests	   = new int[NUM_FLOORS + 1];				//   Note: pFloorRequests[0] unused
		pPassengerList     = new ArrayList();						// list of passenger objects
	}
	
	
    /**
	*
    *	Move elevator one floor in the pTravelDirection.
    *
    */
    public void move() {
        
        pCurrentFloor += pTravelDirection;
            
        // ensure floor is legal
        if (pCurrentFloor < 1) {
            pCurrentFloor    = 1;									// elevator at botton, so stop
            pTravelDirection = 0;
        }
        else if (pCurrentFloor > NUM_FLOORS) {
            pCurrentFloor    = NUM_FLOORS;							// elevator at top, so stop
            pTravelDirection = -1;
        }
		
		System.out.println("Elevator moves to floor " + pCurrentFloor);
    }
	
	
    /**
    * 
    *	Board a passenger destined for a floor.
    * 
    *	@param	objPerson	reference to the Passenger object boarding the elevator.
    */
    public void boardPassenger(Passenger objPerson) throws ElevatorFullException {
			
		if (pPassengerList.size() >= CAPACITY)
			throw new ElevatorFullException("Elevator is full");
		
		pPassengerList.add(objPerson);
		pFloorRequests[objPerson.getDestFloor()]++;
		
		System.out.println("Passenger boards elevator destined for floor " + objPerson.getDestFloor());
    }
	
	
    /**
    * 
    *	Unload all passengers destined for this floor.
	*
	*	The Passenger objects are removed from the Elevators pPassengerList
    *	and given to the Floor object to add to its pResidents list.
	*
    */
    public void unloadPassengers() {
		
		Passenger objPerson;
		Iterator e = pPassengerList.iterator();
		while(e.hasNext()) {
			objPerson = (Passenger) e.next();
			if (objPerson.getDestFloor() == pCurrentFloor) {
				e.remove();											// Person leaves elevator
				objPerson.arrive();									// Person arrives on floor
				objFloor[pCurrentFloor].addPerson(objPerson);
				System.out.println("Passenger leaves elevator on floor " + pCurrentFloor);
			}
		}
    }
	
	
    /**
    * 
    * 	Sets the direction of movement
	*	floorRequests and the destinations passengers waiting on a floor are checked
	*	to determine if the elevator should go up or down.
	*
	*	Note: the current floor is ignored, so if the elevator is full
	*		  and there are still people this floor waiting
	*		  a loop will not occur.
	*
    */
    public void findDirection() {
        
		if (pTravelDirection == 0) {
			pTravelDirection = +1;					// start elevator if not moving
		} else {
		
			boolean switchDirection = true;			// default the Elevator to change direction
		
			// check for passengers on this floor, going in direction of movement.
			if (pTravelDirection == +1 && objFloor[pCurrentFloor].getNumUpPsngrs() > 0)
				switchDirection = false;
			else if (pTravelDirection == -1 && objFloor[pCurrentFloor].getNumDwnPsngrs() > 0)
				switchDirection = false;
			else {
			
				// look for requests in direction of movement.
				for (int i = pCurrentFloor + pTravelDirection; 
				     i > 0 && i <= NUM_FLOORS; i += pTravelDirection) {
					if (pFloorRequests[i] > 0)
						switchDirection = false;
				}
			}
		
			if (switchDirection == true)
				pTravelDirection = 0 - pTravelDirection;
        }
    }
 
 
    /**
    * 
    * 	Check if all elevator requests have been satisfied
	*	If so, true is returned, indicating that all passengers have
	*	arrived at their destinations, so thee elevator has finished
	*	its work and may stop.
	*
	*	@return		true, if all elevator requests have been satisfied.
    */
	public boolean clearedRequests() {
		boolean mReturn = true;
		for (int i = 1; i <= NUM_FLOORS; i++) {
			if (pFloorRequests[i] > 0)
				mReturn = false;
		}
		
		return mReturn;
	}
	
	
    /**
    * 
    * 	Drive elevator to next destination.
    *
    */ 
    public void go() {
        
		do {
		
			findDirection();										// ensure direction of travel
        	if (pFloorRequests[pCurrentFloor] != 0)					// check for floor requests on this floor
				objFloor[pCurrentFloor].unloadAndLoadPassengers(this);
				
			if (clearedRequests() == true)
				pTravelDirection = 0;								// stop elevator if all requests cleared
			else
				move();												// otherwise, move to next floor
			
		} while(pTravelDirection != 0);						
    }
    
        
    /**
    * 
    *	Override the toString() method of java.lang.Object for printing Elevator values.
	*
    *	@return	String		a summary of pertinent values in elevartor, including Passengers.
    */
	public String toString() {
	
		String mReturn = "\n" + "Elevator on floor: " + (pCurrentFloor) + " with "
		                      + pPassengerList.size() + " passengers.\n";
                             
        Passenger objPerson;
		Iterator e = pPassengerList.iterator();
		int counter = 0;
		while(e.hasNext()) {
			objPerson = (Passenger) e.next();
			mReturn += "  <" + ++counter + "> " + objPerson.toString() + "\n";
		}
		
		return mReturn;
	}
        
     
	/**
    * 
    * 	Returns true if elevator is at capacity.
	*	(no longer used, but a nice method to have around)
    *
    *	@return		true if elevator is at capacity.
    */
	public boolean isElevatorFull() {
	
		boolean mReturn = false;
		
		if (pPassengerList.size() == CAPACITY)
			mReturn = true;
			
		return mReturn;
	}
	
	
    /**
    * 
    * 	Needed by loadPassengers() in class Floor to set the floor requests
	*	for floor [mFloor] = the number of passengers still on that floor
	*	waiting for the elevator [mNewValue] when elevator is full.
    *
	*	@param		mFloor		Floor number being checked for.
	*	@param		mNewValue	Floor number being checked for.
    */
	public void setFloorRequests(int mFloor, int mNewValue) {
		pFloorRequests[mFloor] = mNewValue;
	}
	

	/**
    * 
    *	used by unloadAndLoadPassengers() in class Floor to find the direction of travel of
	*	the elevator, and thus determine what passengers should alight the Elevator.
	*
    *	@return		direction of Elevator travel
    */
	public int getTravelDirection() {
		return pTravelDirection;
	}
	
	
    /**
    * 
    *	Register a request for floor [mFloor].
    *
	*	@param		mFloor				Floor number being requested.
	*	@param		mCreateThatPerson	If true put a person on that floor waiting.
    */
	public void registerRequest(int mFloorNum) {
	
		pFloorRequests[mFloorNum]++;								// set a request for floor mFloor
		System.out.println("Elevator requested for floor " + mFloorNum);
	}
	
	
    /**
    * 
    *	Tell class Floor to create a Person object waiting on floor [mFloor]
	*	and wanting to go to Floor [mDestFloor]
    *
	*	@param		mOnFloor		Floor new person is on.
	*	@param		mDestFloor		Floor person wants to go to.
    */
	public void newPerson(int mOnFloor, int mDestFloor) {
		
		objFloor[mOnFloor].addPerson(mDestFloor);					// create the person on the floor waiting
		registerRequest(mOnFloor);
	}
	
	
   /**
    * 
    * 	Create an elevator and board two pPassengers into it.
    *
    */	
	static public void main(String mArgv[]) {

		
		Elevator mElevator = new Elevator();						// make an elevator
		System.out.println(mElevator);								// show starting state of elevator
		
		
		// When calling newPerson, then first parameter is the level the new person is on,
		// the second parameter is the level the person is trying to get to.
		
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(3, 1);
		mElevator.newPerson(6, 1);
		mElevator.newPerson(6, 1);
		mElevator.newPerson(6, 7);
		mElevator.newPerson(6, 2);
		mElevator.newPerson(7, 2);
		mElevator.newPerson(7, 2);
		mElevator.newPerson(7, 2);
		mElevator.newPerson(7, 2);
		mElevator.newPerson(2, 7);
		mElevator.newPerson(2, 7);
		
		mElevator.go();												// Do elevator thing.
		
    }
}

