/**
*
*  ATMProxy.java
*  
*       @author	Russell Lowke
*      @version	Last modified 11_19_03
*
*    CSCIE-160:	Assignment Four
*
*
*  Description:	Methods in ATMProxy do not actually manipulate Account data.
*				Instead, the proxy connects to the server running the real ATM,
*				and for each call sends a message over the network requesting
*				that the server preform the work.
*
*       Method:	Constructor sets up socket via tryToConnect(), 
*				then recieves calls from Client()
*				Most calls to the network are simply sending a string
*				that represents the desired command.  The task of sending
*				the string is handled by networkCall()
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  p   = "property" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw4;

import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

 
public class ATMProxy implements ATM {

	String	pHost;					// location of host
	int		pPort;					// port number used
	
	Socket			pSocket;		// Socket connection to the server
	BufferedReader	pIn  = null; 
	PrintWriter		pOut = null; 

    /**																		   |
    * 
    *	Constructor.
    *
	*	@param	String	The "host" location of the server.
	*	@param	int		The port used by the server.
    */
	public ATMProxy(String mHost, int mPort) {
	
		pHost = mHost;				// assign host location
		pPort = mPort;				// assign port
		
		tryToConnect();
	}
	

    /**																		   |
    * 
    *	Attempt to make a connection to the Server at location [pHost] 
	*	and port [mPort]
    *
    */
    private void tryToConnect() throws IllegalArgumentException {
	
		try {
			System.out.println("Trying to connect to " + pHost + " " + pPort);
			pSocket	= new Socket(pHost, pPort);

			System.out.println("Creating reader on socket " + pPort); 
			pIn = new BufferedReader(
							new InputStreamReader(pSocket.getInputStream()));
			System.out.println("Creating writer on socket " + pPort);
			pOut = new PrintWriter(pSocket.getOutputStream(), true); 
			
		} catch (UnknownHostException e) {
			throw new IllegalArgumentException("Bad host name given.");
		} catch (IOException e) {
			System.out.println("No I/O for connect to localhost: " + e);
			System.exit(1);
		}
		
		System.out.println("Made server connection.");
    }
    
    
    /**																		   |
    * 
    *	Sends a network call to the server asking it to
	*	deposit [amount] into the account.
    *
	*	@param	float	Amount to be deposited.
    */
	public void	deposit(float amount) throws ATMException {
    
		networkCall("deposit("+ amount +")");
    }
    

    /**																		   |
    * 
    *	Sends a network call to the server, asking it to
	*	withdraw [amount] from the account
    *
	*	@param	float	Amount to be withdraw
	*/
    public void withdraw(float amount) throws ATMException {
    
        networkCall("withdraw("+ amount +")");
    }
    
    
    /**																		   |
    * 
    *	Sends a network call to the server requesting the
	*	balance of the account
    *
	*	@return	float	The balance returned by the server.
    */
    public float getBalance() throws ATMException {

        return stringToFloat( networkCall("getBalance()") );
    }
	

    /**																		   |
    * 
    *	Handles the implementation of sending a String [mString] to
	*	the server.  For every String sent to the Server the Server
	*	issues a reply/return String.
    *
	*	@param	String	The string to be sent to the server
	*	@return	String	The response String the Server replied with.
    */
    private String networkCall(String mString) throws ATMException {
    
        String mReturn = "";
		
		// send string...
		pOut.println(mString);
		
		// wait for response...
		try {
			mReturn = pIn.readLine();
		} catch (IOException e) {
			System.out.println("IOException: " + e.getMessage());
		}
		
		// look for any ATMerrors sent and rethrow them.
		StringTokenizer catchError = new StringTokenizer(mReturn, ":");
		if (catchError.countTokens() == 2) {
			if (catchError.nextToken().equals("ERROR")) {
				throw new ATMException(catchError.nextToken());
			}
		}
		
        return mReturn;
    }
    

    /**																		   |
    * 
    *	Cleanup sockets and file IO used by this client.
    *
    */
	public void cleanUp() {
	
		System.out.println("Closing Socket...");
		try { 
			pOut.close();
			pIn.close();
			pSocket.close(); 
		} catch (IOException e) {
			System.out.println("Client ERROR: " + e);
			System.exit(1);
		}
    }
    

    /**																		   |
    * 
    *	Converts a string to a float.
    *
	*	@param	String		The string to convert.
	*	@return	float		A float representation of the given string.
    */
	public static float stringToFloat(String stringObject) {
		return Float.parseFloat(stringObject.trim());
	}
}


