/**
*
*  Server.java
*  
*       @author	Russell Lowke
*      @version	Last modified 11_19_03
*
*    CSCIE-160:	Assignment Four
*
*        Usage:	java Server <port>
*
*
*  Description:	Opens the Server which instatiates ATMImplementation, then
*				listens, waiting for commands from Client/ATMProxy.
*
*       Method:	
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  p   = "property" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.hw4;

import java.net.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class Server {

    int				pPort;
	ATM				pAtm;
	ServerSocket	pServerSocket;
    
    // Fields used for client socket
    Socket			pConnection;
    BufferedReader	pIn;
    PrintWriter		pOut;
	
    /**																		   |
    * 
    *	Constructor.
	*	Initialize port number and the ATM Imp.
	*	Sends the calls to open the Server, listen
	*	and close.
    *
	*	@param	int		Port number this server listens to.
    */
	public Server(int mPort) {
		pPort = mPort;
		pAtm  = new ATMImplementation();
		
		openServer();
		listen();
        closeServer();
	}


    /**																		   |
    * 
    *	Ensure server is opened smoothly.
    */
	public void openServer() {
	
		System.out.println("Connecting server socket to port " + pPort);
		try {
			pServerSocket = new ServerSocket(pPort);
		} catch (IOException e) {
			System.out.println("Failed to connect to port " + pPort);
			System.out.println("ERROR: " + e);
			System.exit(1);
		}
	}
	
	
    /**																		   |
    * 
    *	Loop while listening to Client.
    */
	public void listen() {
	
		boolean mExit = false;
		try {
			while (! mExit) {
			
				System.out.println("Waiting for a client...");
                pConnection = pServerSocket.accept( );
				
				System.out.println("Got a client...");
				mExit = serviceClient();
			}
            
		} catch (IOException e) {
			System.out.println("IO exception while listening for clients.");
			System.out.println("ERROR: " + e);
			System.exit(1);
		}
		
	}
	
	
    /**																		   |
    * 
    *	Ensure reader is opened smoothly
    *
	*	@return	boolean		true returned if problem
    */
    private boolean openReader() {
    
        boolean mReturn = false;
        
		try { 
            // open reader for this client
            System.out.println("Opening reader for client");
            pIn  = new BufferedReader
						(new InputStreamReader(pConnection.getInputStream()));
            if (pIn == null) {
                System.err.println("Unable to get input stream from client");
                mReturn = true;
            }
        } catch (IOException e) {
		    e.printStackTrace();
		    mReturn = true;
		}
        
        return mReturn;
    }
	
    
    /**																		   |
    * 
    *	Ensure writer is opened smoothly
    *
	*	@return	boolean		true returned if problem
    */
    private boolean openWriter() {
    
        boolean mReturn = false;
            
        try {
            // open writer for this client
            System.out.println("Opening writer for client");
            pOut = new PrintWriter(pConnection.getOutputStream(), true);
            if (pOut == null) {
                System.err.println("Unable to get output stream from client");
                pIn.close();
                mReturn = true;
            }
        } catch (IOException e) {
		    e.printStackTrace();
		    mReturn = true;
		}
        
        return mReturn;
    }
    
	
    /**																		   |
    * 
    *	Handle incoming calls from Client
    *
	*	@return	boolean		true returned when Client finished sending Strings
    */
	private boolean serviceClient() {
	
        boolean mExit;
        
        mExit = ( openReader() || openWriter() );
        
        String mLine	= null;
        String mResult	= null;
        
        // read from client
        while (! mExit) {
        
            try {
                mLine = pIn.readLine();
            } catch (IOException e) {
                e.printStackTrace();
            }
            
            if (mLine == null)
                mExit = true;
            else {
                Cmd cmd = parseLine(mLine);
                
                try {
                    mResult = cmd.doIt();
                } catch (ATMException e) {
                    mResult = "ERROR:" + e.getMessage();
                }
                
                pOut.println(mResult);
            }
        }

        closeReaderWriters();
        
		return true;
	}


    /**																		   |
    * 
    *	Ensure readers and writers are closed smoothly
    */
    private void closeReaderWriters() {
        
        // close client reader and writer
        System.out.println("Closing reader for client");
        if (pIn != null) {
            try {
                pIn.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
		System.out.println("Closing writer for client");
        if (pOut != null)
            pOut.close();
    }
    

    /**																		   |
    * 
    *	Parse the command lone into the command name and args
	*	Commands are sent in the format   "methodName(args)"
	*	Much like a Java method.  Later, reflection might be used to trigger
	*	command/methods.
    *
	*	@return	Cmd		Cmd object that represents method to be called.
    */
    public Cmd parseLine(String mLine) {
        
        // trim junk from line
        mLine.trim();
        
        StringTokenizer cmdTknizer = new StringTokenizer(mLine, "(,)");
        int tokenCount = cmdTknizer.countTokens();
        
        String		cmd  = "";
        ArrayList	args = new ArrayList();
        
        
        if (tokenCount > 0) {
        
            // take first token as command/method name
            cmd = cmdTknizer.nextToken();
            
            // append subsequent tokens to the args list
            while(cmdTknizer.hasMoreTokens())
                args.add(cmdTknizer.nextToken().trim());
        }
        
        Cmd mReturn = new BogusCmd(pAtm, args);
        
        if(cmd.equals("deposit"))  {
            mReturn = new DepositCmd(pAtm, args);
        } else if(cmd.equals("withdraw"))  {
            mReturn = new WithdrawCmd(pAtm, args);
        } else if(cmd.equals("getBalance"))  {
            mReturn = new GetBalanceCmd(pAtm);
        }
        
        return mReturn;
    }
    

    /**																		   |
    * 
    *	Ensure Server closed smoothly
    */
	public void closeServer() {

		System.out.println("Closing server socket");
		try {
			pServerSocket.close();				 // close Server Socket
		} catch (IOException e) {
			System.out.println("Failed to close server socket");
			System.out.println("ERROR: " + e);
			System.exit(1);
		}
	}
    
    
    /**																		   |
    * 
    *	Read args passed to main(), and trigger Server to be opened.
    *
	*	@param	argv		Elevator object stopping at this floor.
    */
    public static void main(String[] argv) {
    
		if (argv.length < 1) {
			System.out.println("Usage: java Server <port>");
			System.exit(1);
		}
		
		int mPort = -1;
		try {
			// converts a String to an int
			mPort = Integer.parseInt(argv[0]);
		} catch (NumberFormatException e) { 
			System.out.println(argv[0] + " not a number");
			System.exit(1);
		}
		
		Server mServer = new Server(mPort);
    }
}