/**																			   |
*
*  Security.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*
*  Description:	The Security component is a remote object that has methods for
*				authenticating AccountInfo objects and for authorizing specific
*				operations on individual Accounts.
*
*       Method:	Uses accessors and mutators to modify the field fBalance.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/



package cscie160.project;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;


public class SecurityImpl extends UnicastRemoteObject
        implements Security, ATMParams {
    
    private Bank fBank = null;
    
    /**																		   |
    * 
    *	Constructor.
    *
    */
    public SecurityImpl(Bank mBank) throws RemoteException {
        
        fBank = mBank;	// Security must be able to talk to the Bank object
    }


    /**																		   |
    * 
    *	Send all accounts specified in mTransaction for authentication
    *
    */
    public void authenticateAccounts(TransactionNotification mTransaction)
            throws SecurityException, RemoteException {
        
        // check source/primary account
        authenticateAccount(mTransaction.getSourceAcc());
        
        // if a TRANSFER then also test target account
        if (mTransaction.getTransactionType() == TRANSFER)
            authenticateAccount(mTransaction.getTargetAcc());
    }
    

    /**																		   |
    * 
    *	Validate an account, ensuring it exists and has the correct pin,
    *	otherwise a Security exception is thrown
    *
    *	Note: getAccountNum() can throw a SecurityException, which is simply
    *	passed on.
	*
    */
    private void authenticateAccount(AccountInfo mAccount)
            throws SecurityException, RemoteException {
    
        if (mAccount == null)
            throw new SecurityException("Account information absent");
        
        // get accountInfo data
        int mAcn = mAccount.getAccountNum();
        int mPin = mAccount.getAccountPin();
        
        // get actual account data and pin
        AccountImpl mActualAccount = (AccountImpl) fBank.getAccount(mAcn);
        int mActualPin = mActualAccount.getAccountPin();
        
        // check if submitted pin matches real pin
        if (mPin != mActualPin)
            throw new SecurityException("Incorrect pin for account " +
                                        convertAcnToStr(mAcn) );
    }
    
    
    /**																		   |
    * 
    *	Check authorization on account(s) according to transaction type.
    *	If unauthorized, a SecurityException is thrown
    *
    */
    public void authorizeOperation(TransactionNotification mTransaction)
            throws SecurityException, RemoteException {
        
        int mTransactionType        = mTransaction.getTransactionType();
        AccountInfo mSrcAccountInfo = mTransaction.getSourceAcc();
        
        // use Bank to obtain account reference
        AccountImpl mSrcAccount =
                (AccountImpl) fBank.getAccount(mSrcAccountInfo.getAccountNum());

        switch (mTransactionType) {
            case DEPOSIT:
                authorizeDeposit(mSrcAccount);
                break;
            case WITHDRAWAL:
                authorizeWithdrawal(mSrcAccount);
                break;
            case BALANCEINQUIRY:
                authorizeBalance(mSrcAccount);
                break;
            case TRANSFER:
            
                // need the target account, use Bank to obtain account reference
                AccountInfo mTgtAccountInfo = mTransaction.getTargetAcc();
                AccountImpl mTgtAccount =
                    (AccountImpl) fBank.getAccount(mTgtAccountInfo.getAccountNum());
                
                authorizeTransfer(mSrcAccount, mTgtAccount);
                break;
        }
    }

    
    /**																		   |
    * 
    *	Methods to check authorization on balance, deposit, withdraw,
    *	and transfer.  The actual account is referenced, which exists
    *	localy on the Bank server.
    *
    */
	private void authorizeBalance(AccountImpl mAccount)
            throws SecurityException {
    
        if ( ! mAccount.getAuthBalance() )
            throw new SecurityException( 
                "Not permitted to inquire balance of account " +
                convertAcnToStr(mAccount.getAccountNum()) );
    }
    private void authorizeDeposit(AccountImpl mAccount)
            throws SecurityException {
    
        if ( ! mAccount.getAuthDeposit() )
            throw new SecurityException( 
                "Not permitted to deposit into account " +
                convertAcnToStr(mAccount.getAccountNum()) );
    }
	private void authorizeWithdrawal(AccountImpl mAccount)
            throws SecurityException {

        if ( ! mAccount.getAuthWithdraw() )
            throw new SecurityException(
                "Not permitted to withdraw from account " +
                convertAcnToStr(mAccount.getAccountNum()) );
    }
	private void authorizeTransfer(AccountImpl mSrcAccount, AccountImpl mTgtAccount)
            throws SecurityException {
        
        authorizeWithdrawal(mSrcAccount);
        authorizeDeposit(mTgtAccount);

    }
    
    
	/**																		   |
    *
    *	Convert an account number [int] to a string
    *	with preceeding "0"s.
    *
    *	@return		string version of account numbr with leading "0"s
	*/
    private String convertAcnToStr(int mAccountNum) {
    
        String  mReturn = "" + mAccountNum;
        int    mCounter = ACCOUNTSTRLENGTH - mReturn.length();
        
        while (mCounter > 0) {						// add preceeding "0"s
            mReturn = "0" + mReturn;
            mCounter--;
        }
        
        return mReturn;
    }
}