
// USAGE: java DynamicClassLoading [classname]

//   java DynamicClassLoading DynamicClassLoading

or

//   java DynamicClassLoading SimpleTest

import java.lang.reflect.*;
import java.io.*;

class DynamicClassLoading
{
	/** This program illustrates how, by using reflection, a JVM can be introduced
	*  to classes that were not mentioned in the Java code we are running.  The 
	*  code is very narrowly conceived.  The user supplies the name of a class 
	*  (including its package) as a command line argument.  The program uses the 
	*  class name to get a Class object, uses the Class object to make a new 
	*  instance of the class, then gets the declared methods from the Class 
	*  object.  User is prompted to choose a Method and then prompted for 
	*  input args (assumed to be of type int in this program).  Finally, the 
	*  program invokes the chosen Method using the instance of the class and 
	*  the input arguments.
	*/

	Object createInstance(String className) throws Exception {
        return Class.forName(className).newInstance();
    }

    public void printTest() {
        System.out.println("PRINT TEST WORKED.");
    }
    
    private int getSelection(String prompt)  {
        System.out.print(prompt);
        int result = -1;
        try {
            String response = (new BufferedReader(new InputStreamReader(System.in))).readLine();
            result = Integer.parseInt(response);
        } catch (Exception e) { }
        
        return result;
    }
    
	/** Iterates through array of Method objects, prints out
	*  each method name together with a sequential item number.
	*  User is prompted to choose a Method by item number.
	*  User is then shown the argument signature of the Method
	*  and asked to confirm the choice.  The selected Method is
	*  returned or null, according to user input.
	*/
	private Method chooseMethod(Method [] methods) {
    
        Method result = null;
        String methodNames = "";
        
        // Collect method names
        String methodSignatures [] = new String[methods.length];
        
        // Here we construct a String for each method signature
        for(int i = 0; i < methods.length; i++){
        
            methodSignatures[i] = methods[i].getName() + "(";
            Class [] arguments = methods[i].getParameterTypes();
            int numberOfArgs = arguments.length;
            for(int j = 0; j < numberOfArgs; j++) {
                methodSignatures[i] += arguments[j].getName();
                if(j < (numberOfArgs - 1))
                    methodSignatures[i] += ",";
            }
            methodSignatures[i] += ")";
        }
        
        System.out.println("Select a method:");
        
        // Print out the method signatures.
        for(int k = 0; k < methodSignatures.length; k++)
            System.out.println(k + ": " + methodSignatures[k]);
        int choice = getSelection("Select a method: ");
        if(choice >= 0 && choice < methods.length) 
            result = methods[choice];
            
        return result;
    }

	/** Builds an array of Integers which it returns to caller 
	*/
    private Integer[] getIntegerArray(int numberToGet) throws Exception {
    
        Integer [] result = new Integer[numberToGet];
        for(int i = 0; i < numberToGet; i++) 
            result[i] = new Integer(getSelection("Enter integer value: "));
            
        return result;
    }

    static public void main(String argv[]) throws Exception {
    
        DynamicClassLoading dcl = new DynamicClassLoading();
        
        // Discover the class's methods 
        Class ourSubjectClass = Class.forName(argv[0]);
        
        // getDeclaredMethods 
        Method methods [] = ourSubjectClass.getDeclaredMethods();
        
        // Choose one of them
        Method method = dcl.chooseMethod(methods);
        
        if(method != null)
            System.out.println("Method chosen is named " + method.getName());
            // Trace.trace(1, "Method chosen is named " + method.getName());
        else {
            System.out.println("No method selected, exiting");
            System.exit(4);
        }
    
        // Assuming the class has a no-arg constructor, create an instance of the class
        Object object = dcl.createInstance(argv[0]);
        
        // Sent object.toString() to standard output
        System.out.println("object returned from no-arg constructor: " + object);
        
        int numberOfArgsToChosenMethod = method.getParameterTypes().length;
        // Trace.trace(2, "Number of args for chosen method  = " + numberOfArgsToChosenMethod);
        System.out.println("Number of args for chosen method  = " + numberOfArgsToChosenMethod);
        Integer [] args = dcl.getIntegerArray(numberOfArgsToChosenMethod);
        // Trace.trace(2, "args.length = " + args.length);
        System.out.println("args.length = " + args.length);
        
        // Now run our method with this args array
        method.invoke(object, args);
        
        // Show the object with any side effects of running the method
        System.out.println("object after executing method: " + object);
	}
    
    
    // other reflection examples...
	public void outputFields() {                                                
                                                                                
        Class mClass = Property.class;                                          
                                                                                
        Field[] mFields = mClass.getDeclaredFields();
        String mFldType;
        String mFldName;
        Object mFldVal;
                              
                                                                                
        for (int i = 0; i < mFields.length; i++) {
            try {
            
                mFldType = fixType( mFields[i].getType().getName() );
                mFldName = mFields[i].getName();
                mFldVal  = mFields[i].get(this);
                
                System.out.println("    mFldType = " + mFldType );
                System.out.println("    mFldName = " + mFldName );
                System.out.println("     mFldVal = " + mFldVal );
                System.out.println();

            } catch (Exception e) {
                System.out.println(e.getMessage());
            }          
        }                                                                       
    } 
    
    /*
    // looking for the method requestedMethod
    Method requestedMethod = myClass.getDeclaredMethod(myMethodName, new Class[0]);
    
    try {
        
        Method [] methodList = myClass.getDeclaredMethods();
        int methodIndex = 0;
        for(int i = 0; i < methodList.length; i++) {
        
            if (requestedMethod.equals(methodList[i])) {
                methodIndex = i;
                break;
            }
        }
    } catch (NoSuchMethodExceptione e) { e.printStackTrace(); }
    */

}
