/**																			   |
*
*  ATMImpl.java
*  
*       @author	Russell Lowke
*      @version	Last modified 01_14_04
*
*    CSCIE-160:	Final Project
*
*
*  Description:	The ATM is a remote object that provides various bank operations 
*				by implementing the remote ATM interface. Each operation takes an
*				AccountInfo object as its first parameter along with operation
*				specific parameters. 
*
*				Each banking operation is implemented in the following way:
*
*				- notify all registered listeners of the transaction
*				- use security service to authenticate account info
*				- use security service to authorize operation on account
*				- use bank to obtain account reference(s)
*				- use account reference(s) to perform transaction
*
*				While processing, the ATM tracks its cash on hand (deposits are
*				assumed to be checks and do not added to the available cash). 
*				The ATM begins with $500 cash.
*
*       Method:	Accessors and mutators that redirect call the Account.
*
*		 Notes: Tab Width = 4
*
*				Variable naming prefix conventions:
*				  f   = "field" of object, indicates member variable/field
*				  m   = "my", indicates a local variable to a method
*				  obj = indicates an object from this package
*/
 

package cscie160.project;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.net.MalformedURLException;
import java.util.*;



public class ATMImpl extends java.rmi.server.UnicastRemoteObject 
        implements ATM, ATMParams {
    
	float fHardCash    = 500;
	Bank     fBank     = null;
    Security fSecurity = null;
    
	// Holds listeners who register for all Events produced by this producer
	protected Vector fListeners = new Vector();
    
    ///////
    //
    //  these addListener() and distributeEvent() methods were adapted from 
    // 	EventProducer.java, as found in package babysim
    
    /**																		   |
    *
	*	Listeners interested in Events sign on here
	*
	*/
	public void addListener(ATMListener mListener) throws java.rmi.RemoteException {
        if (! fListeners.contains(mListener)) {
            fListeners.addElement(mListener);
        }
	}
    
    
    /**																		   |
    *
	*	Calls distributeEvent method on every mListener on global-mListener list
	*	get listeners signed on by type and call their distributeEvent method
	*	Note: any mListener on both global list and type-specific list will
	*	get two calls to handleEvent
	*	Note: since multiple remote objects can call methods on a single
	*	EventProducer, we have the possibility of interleaved execution of
	*	this method by multi-threads.  Hence, we clone the listeners before
	*	using them.  Thus, exporting an object for remote access may imply
	*	multi-threaded execution in its JVM without any explicit thread
	*	creation.
    *
    *	@param	EventObject		The event being distributed	
	*/
	public void distributeEvent(EventObject mEvent) throws java.rmi.RemoteException {

        Vector fListenersClone = (Vector) fListeners.clone();
        Enumeration enum = fListenersClone.elements();
        while(enum.hasMoreElements()) {
            ATMListener mElement = (ATMListener) enum.nextElement();
            try {
                mElement.handleEvent(mEvent);
            } catch (Exception e) {
                System.out.println(e); 
                fListeners.removeElement(mElement);
            }
        }
	}
    
    
    /**																		   |
    * 
    *	Constructor.
	*	This ATM has three accounts
    *
    */
    public ATMImpl() throws RemoteException {
        super();
        
        try {
            // lookup the bank
            fBank = (Bank) Naming.lookup("//localhost/bank");
        } catch (java.rmi.NotBoundException e) {
            System.out.println(e.getMessage());
        } catch (java.rmi.RemoteException e) {
            System.out.println(e.getMessage());
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
        }
        
        
        try {
            // lookup security
            fSecurity = (Security) Naming.lookup("//localhost/security");
        } catch (java.rmi.NotBoundException e) {
            System.out.println(e.getMessage());
        } catch (java.rmi.RemoteException e) {
            System.out.println(e.getMessage());
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
        }
    }
    
    
    /**																		   |
    * 
    *	Asks account for the balance of that account.
    *
    *	@param	AccountInfo		Information object on the account requested
    *
	*	@return	float	Balance in [account]
    */
    public float getBalance(AccountInfo mAccountInfo)
                throws RemoteException, SecurityException {
        
        TransactionNotification mTransaction =
                new TransactionNotification(this, BALANCEINQUIRY,
                                           mAccountInfo,
                                           null,
                                           0);
        
        // notify all registered listeners of the transaction
        distributeEvent(mTransaction);
        
        // use security service to authenticate account info
        fSecurity.authenticateAccounts(mTransaction);
        
        // use security service to authorize operation on account(s)
        fSecurity.authorizeOperation(mTransaction);
            
        // use bank to obtain account references
        int mAcnNum = mAccountInfo.getAccountNum();
        
        Account mAccount = fBank.getAccount(mAcnNum);
        
        // use account reference(s) to get balance
        return mAccount.getBalance();
    }
    

    
    /**																		   |
    * 
    *	Tells account to deposit [amount].
    *
    *	@param	AccountInfo		Information object on the account requested
	*	@param	float			Amount to be deposited.
    */
	public void	deposit(AccountInfo mAccountInfo, float mAmount)
                                    throws RemoteException, ATMException, SecurityException {
    
        // all deposits are assumed as checks, so fHardCash is not influenced.
    
        // create transaction instance
        TransactionNotification mTransaction =
                new TransactionNotification(this, DEPOSIT,
                                           mAccountInfo,
                                           null,
                                           mAmount);
                                           
        // notify all registered listeners of the transaction
        distributeEvent(mTransaction);
        
        // use security service to authenticate account info
        fSecurity.authenticateAccounts(mTransaction);
        
        // use security service to authorize operation on account(s)
        fSecurity.authorizeOperation(mTransaction);
        
        // use bank to obtain account references
        Account mAccount = fBank.getAccount(mAccountInfo.getAccountNum());
        
        // use account reference to perform transaction
        mAccount.deposit(mAmount);
    }

	
    /**																		   |
    * 
    *	Tells account to withdraw [amount]
    *
    *	@param	AccountInfo		Information object on the account requested
	*	@param	float			Amount to be withdrawn.
    */
    public void withdraw(AccountInfo mAccountInfo, float mAmount)
                                    throws RemoteException, ATMException, SecurityException {
    
        TransactionNotification mTransaction =
                new TransactionNotification(this, WITHDRAWAL,
                                           mAccountInfo,
                                           null,
                                           mAmount);
        
        // notify all registered listeners of the transaction
        distributeEvent(mTransaction);
        
        // use security service to authenticate account info
        fSecurity.authenticateAccounts(mTransaction);
        
        // use security service to authorize operation on account(s)
        fSecurity.authorizeOperation(mTransaction);
    
        if (fHardCash < mAmount)		// ensure enough hard cash to despense
            throw new ATMException("ATM does not have enough cash.");
        else {
            
            // use bank to obtain account reference
            Account mAccount = fBank.getAccount(mAccountInfo.getAccountNum());
        
            // use account reference to perform transaction
            mAccount.withdraw(mAmount);
        
            // despense hard cash
            fHardCash -= mAmount;
        }
    }

    
	/**																		   |
    * 
    *	Asks account for the balance in that account.
    *
    *	@param	AccountInfo		Information object on the source account
	*	@param	AccountInfo		Information object on the target account
	*	@return	float			Balance in [account]
    */
    public void transfer(AccountInfo mSrcAccountInfo, 
                         AccountInfo mTgtAccountInfo, 
                         float       mAmount)
                                    throws RemoteException, ATMException, SecurityException {
    
        TransactionNotification mTransaction =
                new TransactionNotification(this, TRANSFER,
                                           mSrcAccountInfo,
                                           mTgtAccountInfo,
                                           mAmount);
                                           
        // notify all registered listeners of the transaction
        distributeEvent(mTransaction);
        
        // use security service to authenticate account info
        fSecurity.authenticateAccounts(mTransaction);
        
        // use security service to authorize operation on account(s)
        fSecurity.authorizeOperation(mTransaction);
        
        // use bank to obtain account references
        Account mSrcAccount = fBank.getAccount(mSrcAccountInfo.getAccountNum());
        Account mTgtAccount = fBank.getAccount(mTgtAccountInfo.getAccountNum());
        
        // use account reference to perform transaction
        mSrcAccount.transfer(mAmount, mTgtAccount);
    }
}